@@ -38,16 +38,7 @@
    A FILE of `-' means standard input.
    If no FILEs are given, standard input is used. */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
 
 #include <stdio.h>
 #include <getopt.h>
@@ -145,7 +136,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("paste - %s\n", version_string);
       exit (0);
     }
 
@@ -257,6 +248,10 @@ paste_parallel (nfiles, fnamptr)
   int i;			/* Loop index. */
   int opened_stdin = 0;		/* Nonzero if any fopen got fd 0. */
 
+#ifdef lint  /* Suppress `used before initialized' warning.  */
+  chr = 0;
+#endif
+
   delbuf = (char *) xmalloc (file_list_size + 2);
   fileptr = (FILE **) xmalloc ((file_list_size + 1) * sizeof (FILE *));
 
