@@ -94,16 +94,7 @@
    -w width	Set the page width to WIDTH characters. */
 
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
 
 #include <stdio.h>
 #include <getopt.h>
@@ -625,7 +616,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("pr - %s\n", version_string);
       exit (0);
     }
 
