@@ -21,16 +21,7 @@
    * Implement -t CHAR or -t REGEX to specify break characters other
      than newline. */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
 
 #include <stdio.h>
 #include <getopt.h>
@@ -40,6 +31,8 @@
 
 char *xmalloc ();
 void error ();
+int full_write ();
+int safe_read ();
 
 static int convint ();
 static int isdigits ();
@@ -217,7 +210,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("split - %s\n", version_string);
       exit (0);
     }
 
@@ -524,7 +517,7 @@ cwrite (new_file_flag, bp, bytes)
       if (output_desc < 0)
 	error (1, errno, "%s", outfile);
     }
-  if (write (output_desc, bp, bytes) < 0)
+  if (full_write (output_desc, bp, bytes) < 0)
     error (1, errno, "%s", outfile);
 }
 
@@ -542,7 +535,7 @@ stdread (buf, nchars)
 
   while (to_be_read)
     {
-      n_read = read (input_desc, buf, to_be_read);
+      n_read = safe_read (input_desc, buf, to_be_read);
       if (n_read < 0)
 	return -1;
       if (n_read == 0)
