@@ -35,16 +35,7 @@
 tac -r -s '.\|
 ' file */
 
-#ifdef HAVE_CONFIG_H
-#if defined (CONFIG_BROKETS)
-/* We use <config.h> instead of "config.h" so that a compilation
-   using -I. -I$srcdir will use ./config.h rather than $srcdir/config.h
-   (which it would do because it found this file in $srcdir).  */
 #include <config.h>
-#else
-#include "config.h"
-#endif
-#endif
 
 #include <stdio.h>
 #include <getopt.h>
@@ -78,6 +69,8 @@ static void save_stdin ();
 static void xwrite ();
 
 void error ();
+int full_write ();
+int safe_read ();
 
 /* The name this program was run with. */
 char *program_name;
@@ -145,7 +138,7 @@ Usage: %s [OPTION]... [FILE]...\n\
       printf ("\
 \n\
   -b, --before             attach the separator before instead of after\n\
-  -r, --regex              intepret the separator as a regular expression\n\
+  -r, --regex              interpret the separator as a regular expression\n\
   -s, --separator=STRING   use STRING as the separator instead of newline\n\
       --help               display this help and exit\n\
       --version            output version information and exit\n\
@@ -196,7 +189,7 @@ main (argc, argv)
 
   if (show_version)
     {
-      printf ("%s\n", version_string);
+      printf ("tac - %s\n", version_string);
       exit (0);
     }
 
@@ -382,8 +375,8 @@ save_stdin ()
       error (0, errno, "%s", tempfile);
       cleanup ();
     }
-  while ((bytes_read = read (0, buffer, read_size)) > 0)
-    if (write (fd, buffer, bytes_read) != bytes_read)
+  while ((bytes_read = safe_read (0, buffer, read_size)) > 0)
+    if (full_write (fd, buffer, bytes_read) < 0)
       {
 	error (0, errno, "%s", tempfile);
 	cleanup ();
@@ -409,7 +402,7 @@ tac_file (file)
 {
   int fd, errors;
 
-  fd = open (file, 0);
+  fd = open (file, O_RDONLY);
   if (fd == -1)
     {
       error (0, errno, "%s", file);
@@ -466,9 +459,9 @@ tac (fd, file)
      in the input file. */
 
   lseek (fd, file_pos, SEEK_SET);
-  if (read (fd, buffer, saved_record_size) != saved_record_size)
+  if (safe_read (fd, buffer, saved_record_size) != saved_record_size)
     {
-      error (0, 1, "%s", file);
+      error (0, errno, "%s", file);
       return 1;
     }
 
@@ -562,7 +555,7 @@ tac (fd, file)
 	  else
 	    match_start = past_end;
 
-	  if (read (fd, buffer, read_size) != read_size)
+	  if (safe_read (fd, buffer, read_size) != read_size)
 	    {
 	      error (0, errno, "%s", file);
 	      return 1;
@@ -640,7 +633,7 @@ xwrite (desc, buffer, size)
      char *buffer;
      int size;
 {
-  if (write (desc, buffer, size) != size)
+  if (full_write (desc, buffer, size) < 0)
     {
       error (0, errno, "write error");
       cleanup ();
