@@ -45,18 +45,35 @@
    Extensions by David MacKenzie <djm@gnu.ai.mit.edu>.
    tail -f for multiple files by Ian Lance Taylor <ian@airs.com>.  */
 
+#include <config.h>
+
 #include <stdio.h>
+#include <assert.h>
 #include <getopt.h>
 #include <sys/types.h>
-#include <signal.h>
 #include "system.h"
 #include "version.h"
 
+/* Disable assertions.  Some systems have broken assert macros.  */
+#define NDEBUG 1
+
+#define XWRITE(fd, buffer, n_bytes)					\
+  do									\
+    {									\
+      assert ((fd) == 1);						\
+      assert ((n_bytes) >= 0);						\
+      if (n_bytes > 0 && fwrite ((buffer), 1, (n_bytes), stdout) == 0)	\
+	error (1, errno, "write error");				\
+    }									\
+  while (0)
+
 /* Number of items to tail. */
 #define DEFAULT_NUMBER 10
 
 /* Size of atomic reads. */
+#ifndef BUFSIZE
 #define BUFSIZE (512 * 8)
+#endif
 
 /* Number of bytes per item we are printing.
    If 0, tail in lines. */
@@ -87,8 +104,8 @@ enum header_mode
 };
 
 char *xmalloc ();
-void xwrite ();
 void error ();
+int safe_read ();
 
 static int file_lines ();
 static int pipe_bytes ();
@@ -113,10 +130,10 @@ char *program_name;
 static int have_read_stdin;
 
 /* If non-zero, display usage information and exit.  */
-static int flag_help;
+static int show_help;
 
-/* If non-zero, print the version on standard error.  */
-static int flag_version;
+/* If non-zero, print the version on standard output then exit.  */
+static int show_version;
 
 static struct option const long_options[] =
 {
@@ -126,8 +143,8 @@ static struct option const long_options[] =
   {"quiet", no_argument, NULL, 'q'},
   {"silent", no_argument, NULL, 'q'},
   {"verbose", no_argument, NULL, 'v'},
-  {"help", no_argument, &flag_help, 1},
-  {"version", no_argument, &flag_version, 1},
+  {"help", no_argument, &show_help, 1},
+  {"version", no_argument, &show_version, 1},
   {NULL, 0, NULL, 0}
 };
 
@@ -201,7 +218,7 @@ main (argc, argv)
 
 		default:
 		  error (0, 0, "unrecognized option `-%c'", *argv[1]);
-		  usage ();
+		  usage (1);
 		}
 	      ++argv[1];
 	    }
@@ -252,18 +269,18 @@ main (argc, argv)
 	  break;
 
 	default:
-	  usage ();
+	  usage (1);
 	}
     }
 
-  if (flag_version)
+  if (show_version)
     {
-      fprintf (stderr, "%s\n", version_string);
+      printf ("tail - %s\n", version_string);
       exit (0);
     }
 
-  if (flag_help)
-    usage ();
+  if (show_help)
+    usage (0);
 
   if (number == -1)
     number = DEFAULT_NUMBER;
@@ -305,7 +322,7 @@ main (argc, argv)
 
   if (have_read_stdin && close (0) < 0)
     error (1, errno, "-");
-  if (close (1) < 0)
+  if (fclose (stdout) == EOF)
     error (1, errno, "write error");
   exit (exit_status);
 }
@@ -329,7 +346,7 @@ tail_file (filename, number, filenum)
       have_read_stdin = 1;
       filename = "standard input";
       if (print_headers)
-	write_header (filename);
+	write_header (filename, NULL);
       errors = tail (filename, 0, number);
       if (forever_multiple)
 	{
@@ -367,7 +384,7 @@ tail_file (filename, number, filenum)
       else
 	{
 	  if (print_headers)
-	    write_header (filename);
+	    write_header (filename, NULL);
 	  errors = tail (filename, fd, number);
 	  if (forever_multiple)
 	    {
@@ -407,20 +424,16 @@ tail_file (filename, number, filenum)
 }
 
 static void
-write_header (filename)
-     char *filename;
+write_header (filename, comment)
+     const char *filename;
+     const char *comment;
 {
   static int first_file = 1;
 
-  if (first_file)
-    {
-      xwrite (1, "==> ", 4);
-      first_file = 0;
-    }
-  else
-    xwrite (1, "\n==> ", 5);
-  xwrite (1, filename, strlen (filename));
-  xwrite (1, " <==\n", 5);
+  printf ("%s==> %s%s%s <==\n", (first_file ? "" : "\n"), filename,
+	  (comment ? ": " : ""),
+	  (comment ? comment : ""));
+  first_file = 0;
 }
 
 /* Display the last NUMBER units of file FILENAME, open for reading
@@ -558,7 +571,7 @@ file_lines (filename, fd, number, pos)
      reads will be on block boundaries, which might increase efficiency. */
   pos -= bytes_read;
   lseek (fd, pos, SEEK_SET);
-  bytes_read = read (fd, buffer, bytes_read);
+  bytes_read = safe_read (fd, buffer, bytes_read);
   if (bytes_read == -1)
     {
       error (0, errno, "%s", filename);
@@ -580,7 +593,7 @@ file_lines (filename, fd, number, pos)
 	      /* If this newline wasn't the last character in the buffer,
 	         print the text after it. */
 	      if (i != bytes_read - 1)
-		xwrite (1, &buffer[i + 1], bytes_read - (i + 1));
+		XWRITE (1, &buffer[i + 1], bytes_read - (i + 1));
 	      return 0;
 	    }
 	}
@@ -594,7 +607,7 @@ file_lines (filename, fd, number, pos)
       pos -= BUFSIZE;
       lseek (fd, pos, SEEK_SET);
     }
-  while ((bytes_read = read (fd, buffer, BUFSIZE)) > 0);
+  while ((bytes_read = safe_read (fd, buffer, BUFSIZE)) > 0);
   if (bytes_read == -1)
     {
       error (0, errno, "%s", filename);
@@ -632,7 +645,7 @@ pipe_lines (filename, fd, number)
   tmp = (LBUFFER *) xmalloc (sizeof (LBUFFER));
 
   /* Input is always read into a fresh buffer. */
-  while ((tmp->nbytes = read (fd, tmp->buffer, BUFSIZE)) > 0)
+  while ((tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZE)) > 0)
     {
       tmp->nlines = 0;
       tmp->next = NULL;
@@ -711,10 +724,10 @@ pipe_lines (filename, fd, number)
     }
   else
     i = 0;
-  xwrite (1, &tmp->buffer[i], tmp->nbytes - i);
+  XWRITE (1, &tmp->buffer[i], tmp->nbytes - i);
 
   for (tmp = tmp->next; tmp; tmp = tmp->next)
-    xwrite (1, tmp->buffer, tmp->nbytes);
+    XWRITE (1, tmp->buffer, tmp->nbytes);
 
 free_lbuffers:
   while (first)
@@ -754,7 +767,7 @@ pipe_bytes (filename, fd, number)
   tmp = (CBUFFER *) xmalloc (sizeof (CBUFFER));
 
   /* Input is always read into a fresh buffer. */
-  while ((tmp->nbytes = read (fd, tmp->buffer, BUFSIZE)) > 0)
+  while ((tmp->nbytes = safe_read (fd, tmp->buffer, BUFSIZE)) > 0)
     {
       tmp->next = NULL;
 
@@ -808,10 +821,10 @@ pipe_bytes (filename, fd, number)
     i = total_bytes - number;
   else
     i = 0;
-  xwrite (1, &tmp->buffer[i], tmp->nbytes - i);
+  XWRITE (1, &tmp->buffer[i], tmp->nbytes - i);
 
   for (tmp = tmp->next; tmp; tmp = tmp->next)
-    xwrite (1, tmp->buffer, tmp->nbytes);
+    XWRITE (1, tmp->buffer, tmp->nbytes);
 
 free_cbuffers:
   while (first)
@@ -836,7 +849,7 @@ start_bytes (filename, fd, number)
   char buffer[BUFSIZE];
   int bytes_read = 0;
 
-  while (number > 0 && (bytes_read = read (fd, buffer, BUFSIZE)) > 0)
+  while (number > 0 && (bytes_read = safe_read (fd, buffer, BUFSIZE)) > 0)
     number -= bytes_read;
   if (bytes_read == -1)
     {
@@ -844,7 +857,7 @@ start_bytes (filename, fd, number)
       return 1;
     }
   else if (number < 0)
-    xwrite (1, &buffer[bytes_read + number], -number);
+    XWRITE (1, &buffer[bytes_read + number], -number);
   return 0;
 }
 
@@ -862,7 +875,7 @@ start_lines (filename, fd, number)
   int bytes_read = 0;
   int bytes_to_skip = 0;
 
-  while (number && (bytes_read = read (fd, buffer, BUFSIZE)) > 0)
+  while (number && (bytes_read = safe_read (fd, buffer, BUFSIZE)) > 0)
     {
       bytes_to_skip = 0;
       while (bytes_to_skip < bytes_read)
@@ -875,7 +888,7 @@ start_lines (filename, fd, number)
       return 1;
     }
   else if (bytes_to_skip < bytes_read)
-    xwrite (1, &buffer[bytes_to_skip], bytes_read - bytes_to_skip);
+    XWRITE (1, &buffer[bytes_to_skip], bytes_read - bytes_to_skip);
   return 0;
 }
 
@@ -894,9 +907,9 @@ dump_remainder (filename, fd)
 
   total = 0;
 output:
-  while ((bytes_read = read (fd, buffer, BUFSIZE)) > 0)
+  while ((bytes_read = safe_read (fd, buffer, BUFSIZE)) > 0)
     {
-      xwrite (1, buffer, bytes_read);
+      XWRITE (1, buffer, bytes_read);
       total += bytes_read;
     }
   if (bytes_read == -1)
@@ -948,12 +961,24 @@ tail_forever (names, nfiles)
 
 	  /* This file has changed size.  Print out what we can, and
 	     then keep looping.  */
+
+	  changed = 1;
+
+	  if (stats.st_size < file_sizes[i])
+	    {
+	      write_header (names[i], "file truncated");
+	      last = i;
+	      lseek (file_descs[i], stats.st_size, SEEK_SET);
+	      file_sizes[i] = stats.st_size;
+	      continue;
+	    }
+
 	  if (i != last)
 	    {
-	      write_header (names[i]);
+	      if (print_headers)
+		write_header (names[i], NULL);
 	      last = i;
 	    }
-	  changed = 1;
 	  file_sizes[i] += dump_remainder (names[i], file_descs[i]);
 	}
 
@@ -1004,12 +1029,35 @@ atou (str)
 }
 
 static void
-usage ()
+usage (status)
+     int status;
 {
-  fprintf (stderr, "\
-Usage: %s [-c [+]N[bkm]] [-n [+]N] [-fqv] [--bytes=[+]N[bkm]] [--lines=[+]N]\n\
-       [--follow] [--quiet] [--silent] [--verbose] [--help] [--version]\n\
-       [file...]\n\
-       %s [{-,+}Nbcfklmqv] [file...]\n", program_name, program_name);
-  exit (1);
+  if (status != 0)
+    fprintf (stderr, "Try `%s --help' for more information.\n",
+	     program_name);
+  else
+    {
+      printf ("\
+Usage: %s [OPTION]... [FILE]...\n\
+",
+	      program_name);
+      printf ("\
+\n\
+  -c, --bytes=SIZE         print last SIZE bytes\n\
+  -f, --follow             print files as they grow\n\
+  -l, -n, --lines=NUMBER   print last NUMBER lines, instead of last 10\n\
+  -q, --quiet, --silent    never print headers giving file names\n\
+  -v, --verbose            always print headers giving file names\n\
+      --help               display this help and exit\n\
+      --version            output version information and exit\n\
+\n\
+SIZE may have a multiplier suffix: b for 512, k for 1K, m for 1 Meg.\n\
+If SIZE is prefixed by +, prints all except the first SIZE bytes.  If\n\
+NUMBER is prefixed by +, prints all except the first NUMBER lines.  If\n\
+-VALUE or +VALUE is used as first OPTION, read -c VALUE or -c +VALUE\n\
+when one of multipliers bkm follows concatenated, else read -n VALUE\n\
+or -n +VALUE.  With no FILE, or when FILE is -, read standard input.\n\
+");
+    }
+  exit (status);
 }
